<?php 

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\Util\Functions;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Services;

class ImportacaoConsumidorController extends CrudWithParameterBag {
    
    protected $dataSourceName = 'select_layout_importacao_consumidor'; 
    protected $util;
    protected $impConsumidorService;
    protected $impConsumidorProcService;
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag,  $util, Services\ImportacaoConsumidorService $impConsumidorService, Services\ImportacaoConsumidorProcessaService $impConsumidorProcService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this -> util = $util;
        $this->impConsumidorService = $impConsumidorService;
        $this->impConsumidorProcService = $impConsumidorProcService;
    }
    
     
    public function findLayoutConsumidor(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'select_layout_importacao_consumidor';
    	parent::find($request, $response);
    }
    
    public function findClienteLayout(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'select_cliente_layout';
        $this->parameterBagColumns = array('CDLAYOUTIMP');
        parent::find($request, $response);
    }
    
    public function findConsumidorLayout(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'select_consumidor_layout';
        $this->parameterBagColumns = array('CDLAYOUTIMP');
        parent::find($request, $response);
    }
    
    public function findAjustaSaldoParam(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'select_ajusta_saldo_importacao';
        $this->parameterBagColumns = array('CDFILIAL');
        parent::find($request, $response);
    }
    
    public function buildData(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $codClienteInformado = null;
        
        if (isset($conditions['CDCLIENTE']) && isset($conditions['INFORMADO']) && ($conditions['INFORMADO'] == '%S%' ||  $conditions['INFORMADO'] == 'S')) {
            $codClienteInformado = $conditions['CDCLIENTE'];
        }
        
        $layoutFeatures = $this->impConsumidorService->findLayoutFeatures($conditions['CDLAYOUTIMP']);
        $columnsAndPositions = $this->impConsumidorService->findColumnsAndPositions($conditions['CDLAYOUTIMP']);
        
        $decodedFile = base64_decode($conditions['ARQUIVOBASE64']);
        $filename = $conditions['ARQUIVOBASE64NAME'];
        
        if (isset($layoutFeatures[0]) && !empty($layoutFeatures[0])){
            $layoutFeatures = $layoutFeatures[0];
        }
        
        $valLayoutData = $this->impConsumidorService->getValLayout($layoutFeatures['CDLAYOUTIMP'], $layoutFeatures['NRSEQTABELAY']);
        
        $records = $this->impConsumidorService->processFile($decodedFile, $columnsAndPositions, $layoutFeatures, $codClienteInformado, $valLayoutData, $filename);
        
        $response->addDataSet(new DataSet('monta_dados_layout_consumidor', $records));
    }
    
    public function checkAjustaSaldo (DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $IDAJUSALDIMPCON = $this->impConsumidorProcService->findAjustaSaldo($conditions['CDFILIAL']);
        
        if ($IDAJUSALDIMPCON == 'S') {
            $message = '';
            if ($conditions['AJUSTASALDOCHECK']) {
                $message = 'O ajuste de saldo mensal será realizado como primeiro movimento do dia e SOMENTE para os novos consumidores. Confirma a operação?';
            } else {
                $message = 'O ajuste de saldo mensal será realizado como primeiro movimento do dia e para TODOS os consumidores no arquivo. Confirma a operação?';
            }
            
            $response->addMessage(new Message($message));
        }
    }
    
    public function gerarLog (DTO\Request\DataSet $request, DTO\Response $response) {
        $conditions = $this->util->getParamsAsArray($request);
        
        $consumers = array_map(function($consumer) {
            return $consumer['fullRecord'];
        }, $conditions);
        
        $filial = isset($consumers[0]) ? $consumers[0]['CDFILIAL'] : null;
        $IDPARALISAIMPARQ = isset($consumers[0]) ? $consumers[0]['IDPARALISAIMPARQ'] : null;

        list($log, $boParaImp, $boInclui_Log, $consumers) = $this->impConsumidorProcService->verificarDuplicidade($consumers, $filial);
        $response->addDataSet(new DataSet('consumidorLog', $log));
        $response->addDataSet(new DataSet('boParaImp', array('boParaImp' => $boParaImp)));
        $response->addDataSet(new DataSet('boInclui_Log', array('boInclui_Log' => $boInclui_Log)));
        $response->addDataSet(new DataSet('IDPARALISAIMPARQ', array('IDPARALISAIMPARQ' => $IDPARALISAIMPARQ)));
        $response->addDataSet(new DataSet('consumers', array('consumers' => $consumers)));
    } 
    
    public function setConsumidor (DTO\Request\DataSet $request, DTO\Response $response) {
        $conditions = $this->util->getParamsAsArray($request);
        
        $consumers = isset($conditions[0]) ? $conditions[0] : null;
        
        $result = $this->impConsumidorProcService->setConsumidor($consumers);

        $response->addDataSet(new DataSet('setConsumidor', $result));
    }
}